/**
 * Discrete/Modular exponentiation
 *
 * Highly memory efficient because the full result is never stored, but shortened by defined modulo instead.
 * We can use that because the prime divisor required for our algorithm is guarenteed to be smaller
 * than n.
 *
 * Counterpart function to the discrete logarithm.
 */
pub fn mod_pow(base: u32, exp: u32, r#mod: u32) -> u32 {
    let mut result: u64 = 1;
    for _ in 0..exp - 1 {
        result = (result * u64::from(base)) % u64::from(r#mod);
    }
    u32::try_from(result).unwrap()
}

/**
 * very primitive prime checker
 */
pub fn is_prime(n: u32) -> bool {
    for i in (3..=(n as f32).sqrt() as u32).step_by(2) {
        if n % i == 0 {
            return false;
        }
    }
    true
}
